import React from "react";
import { AutoControlledPureComponent, getAutoControlledStateFromProps } from "../../../shared";
import { DatePickerAnchor } from "../date-picker-anchor";
import { POSITIONS } from "../../../popup";
import { SingleDatePickerButtons } from "./single-date-picker-buttons";
import { SingleDatePickerCalendar } from "./single-date-picker-calendar";
import { SingleDatePickerInput } from "./single-date-picker-input";
import { arrayOf, bool, func, node, number, object, oneOf, oneOfType, string } from "prop-types";
import { cloneElement } from "react";
import { isNil } from "lodash";
import { momentObj as momentType } from "react-moment-proptypes";
import moment from "moment"; // Sizes constants are duplicated here until https://github.com/reactjs/react-docgen/pull/352 is merged. Otherwise it will not render properly in the docs.

const SIZES = ["small", "medium", "large"];
export const SINGLE_DATE_PICKER_PROP_TYPES = {
  /**
   * A controlled date value.
   */
  date: momentType,

  /**
   * The initial value of date.
   */
  defaultDate: momentType,

  /**
   * Called when the date is applied.
   * @param {SyntheticEvent} event - React's original SyntheticEvent.
   * @param {Moment} date - Selected date.
   * @param {Object} props - All the props.
   * @returns {void}
   */
  onDateChange: func.isRequired,

  /**
   * Called when the calendar open / close.
   * @param {SyntheticEvent} event - React's original SyntheticEvent.
   * @param {boolean} isVisible - Indicate if the calendar is visible.
   * @param {Object} props - All the props.
   * @returns {void}
   */
  onVisibilityChange: func,

  /**
   * Whether or not the calendar selected date(s) can be cleared.
   */
  allowClear: bool,

  /**
   * The minimum (inclusive) date available for selection.
   */
  minDate: momentType,

  /**
   * The maximum (inclusive) date available for selection.
   */
  maxDate: momentType,

  /**
   * An initial visible month displayed when the calendar open.
   */
  initialVisibleMonth: oneOfType([momentType, func]),

  /**
   * The number of months displayed simultaneously in the calendar.
   */
  numberOfMonths: number,

  /**
   * A React component that display the currently applied date and open the calendar.
   */
  input: node,

  /**
   * The placeholder text.
   */
  placeholder: string,

  /**
   * A format to display a date.
   */
  dateFormat: string,

  /**
   * A position for the calendar.
   */
  position: oneOf(POSITIONS),

  /**
   * An array containing an horizontal and vertical offsets for the calendar position.
   * Ex: ["10px", "-10px"]
   */
  offsets: arrayOf(string),

  /**
   * z-index of the calendar.
   */
  zIndex: string,

  /**
   * A React component to select a date.
   */
  calendar: node,

  /**
   * A React component displayed under the calendar to `clear` and `apply` the date(s).
   */
  buttons: node,

  /**
   * A controlled open value that determined whether or not the calendar is displayed.
   */
  open: bool,

  /**
   * The initial value of open.
   */
  defaultOpen: bool,

  /**
   * Whether or not the calendar should close when the date picker loose focus.
   */
  closeOnBlur: bool,

  /**
   * Whether or not the calendar should close when a click happens outside the date picker.
   * Requires `closeOnBlur` to be false.
   */
  closeOnOutsideClick: bool,

  /**
   * A disabled date picker does not allow user interaction.
   */
  disabled: bool,

  /**
   * Whether or not the date picker take up the width of its container.
   */
  fluid: bool,

  /**
   * A date picker can vary in sizes.
   */
  size: oneOf(SIZES),

  /**
   * Additional classes.
   */
  className: string,

  /**
   * Custom inline style.
   */
  style: object
};
export class SingleDatePicker extends AutoControlledPureComponent {
  constructor(...args) {
    super(...args);
    this.state = {
      date: null,
      selectedDate: null,
      open: false
    };

    this.handleAnchorVisibilityChange = (event, shouldOpen) => {
      const {
        date
      } = this.state;

      if (shouldOpen) {
        this.openCalendar(event);
      } else {
        this.setState({
          selectedDate: date
        });
        this.closeCalendar(event);
      }
    };

    this.handleInputClear = event => {
      const {
        onDateChange
      } = this.props;
      this.trySetAutoControlledStateValue({
        date: null
      });
      this.setState({
        selectedDate: null
      });
      onDateChange(event, null, this.props);
    };

    this.handleCalendarDateChange = date => {
      this.setState({
        selectedDate: date
      });
    };

    this.handleCalendarApply = event => {
      const {
        onDateChange
      } = this.props;
      const {
        selectedDate
      } = this.state;
      this.closeCalendar(event);
      this.trySetAutoControlledStateValue({
        date: selectedDate
      });
      onDateChange(event, selectedDate, this.props);
    };
  }

  static getDerivedStateFromProps(props, state) {
    return getAutoControlledStateFromProps(props, state, SingleDatePicker.autoControlledProps, ({
      date
    }) => ({
      selectedDate: date
    }));
  }

  openCalendar(event) {
    const {
      onVisibilityChange
    } = this.props;
    this.trySetAutoControlledStateValue({
      open: true
    });

    if (!isNil(onVisibilityChange)) {
      onVisibilityChange(event, true, this.props);
    }
  }

  closeCalendar(event) {
    const {
      onVisibilityChange
    } = this.props;
    this.trySetAutoControlledStateValue({
      open: false
    });

    if (!isNil(onVisibilityChange)) {
      onVisibilityChange(event, false, this.props);
    }
  }

  renderInput() {
    const {
      input,
      allowClear,
      numberOfMonths,
      placeholder,
      dateFormat,
      disabled,
      fluid,
      size
    } = this.props;
    const {
      selectedDate
    } = this.state;
    return cloneElement(input, {
      date: selectedDate,
      onClear: this.handleInputClear,
      allowClear,
      numberOfMonths,
      placeholder,
      dateFormat,
      disabled,
      fluid,
      size
    });
  }

  renderCalendar() {
    const {
      allowClear,
      minDate,
      maxDate,
      initialVisibleMonth,
      numberOfMonths,
      calendar,
      buttons
    } = this.props;
    const {
      selectedDate
    } = this.state;
    return cloneElement(calendar, {
      // Since 21.1.0 react-dates mutate the date moment objects. Not sure where and why.
      // To prevent side effects, we provide a clone. https://momentjs.com/docs/#/parsing/moment-clone/
      date: isNil(selectedDate) ? selectedDate : moment(selectedDate),
      onDateChange: this.handleCalendarDateChange,
      onApply: this.handleCalendarApply,
      allowClear,
      minDate,
      maxDate,
      initialVisibleMonth,
      numberOfMonths,
      buttons
    });
  }

  render() {
    const {
      position,
      offsets,
      zIndex,
      disabled,
      closeOnBlur,
      closeOnOutsideClick,
      fluid,
      className,
      style
    } = this.props;
    const {
      open
    } = this.state;
    return /*#__PURE__*/React.createElement(DatePickerAnchor, {
      open: open,
      input: this.renderInput(),
      calendar: this.renderCalendar(),
      position: position,
      offsets: offsets,
      zIndex: zIndex,
      onVisibilityChange: this.handleAnchorVisibilityChange,
      disabled: disabled,
      closeOnBlur: closeOnBlur,
      closeOnOutsideClick: closeOnOutsideClick,
      fluid: fluid,
      className: className,
      style: style
    });
  }

}
SingleDatePicker.displayName = "SingleDatePicker";
SingleDatePicker.propTypes = SINGLE_DATE_PICKER_PROP_TYPES;
SingleDatePicker.defaultProps = {
  input: /*#__PURE__*/React.createElement(SingleDatePickerInput, null),
  allowClear: true,
  dateFormat: "MMM Do YYYY",
  numberOfMonths: 1,
  calendar: /*#__PURE__*/React.createElement(SingleDatePickerCalendar, null),
  buttons: /*#__PURE__*/React.createElement(SingleDatePickerButtons, null),
  disabled: false,
  fluid: false
};
SingleDatePicker.autoControlledProps = ["date", "open"];
SingleDatePicker.Input = SingleDatePickerInput;
SingleDatePicker.Calendar = SingleDatePickerCalendar;
SingleDatePicker.Buttons = SingleDatePickerButtons;